@@ -21,5 +21,25 @@
 #include <glib.h>
 #include <gio/gio.h>
 
+typedef enum _ScreenCastSourceType
+{
+  SCREEN_CAST_SOURCE_TYPE_MONITOR = 1,
+  SCREEN_CAST_SOURCE_TYPE_WINDOW = 2,
+} ScreenCastSourceType;
+
+typedef enum _ScreenCastCursorMode
+{
+  SCREEN_CAST_CURSOR_MODE_NONE = 0,
+  SCREEN_CAST_CURSOR_MODE_HIDDEN = 1,
+  SCREEN_CAST_CURSOR_MODE_EMBEDDED = 2,
+  SCREEN_CAST_CURSOR_MODE_METADATA = 4,
+} ScreenCastCursorMode;
+
+typedef struct _ScreenCastSelection
+{
+  gboolean multiple;
+  ScreenCastCursorMode cursor_mode;
+} ScreenCastSelection;
+
 gboolean screen_cast_init (GDBusConnection *connection,
                            GError **error);
